@@ -2,8 +2,9 @@ import time
 import logging
 from datetime import timedelta, datetime
 
-from pulsar.utils.py2py3 import itervalues, iteritems
-from pulsar.utils.timeutils import remaining, timedelta_seconds, humanize_seconds
+from pulsar.utils.httpurl import itervalues, iteritems
+from pulsar.utils.timeutils import remaining, timedelta_seconds,\
+                                     humanize_seconds
 
 from .models import registry
 from .exceptions import SchedulingError, TaskNotAvailable
@@ -18,13 +19,12 @@ EMPTY_DICT = {}
 
 
 def get_datetime(expiry, start):
-    if expiry:
-        if isinstance(expiry,datetime):
-            return expiry
-        elif isinstance(expiry,timedelta):
-            return start + expiry
-        else:
-            return datetime.fromtimestamp(expiry)
+    if isinstance(expiry, datetime):
+        return expiry
+    elif isinstance(expiry, timedelta):
+        return start + expiry
+    else:
+        return datetime.fromtimestamp(expiry)
 
 
 class Schedule(object):
@@ -33,11 +33,11 @@ class Schedule(object):
         self.run_every = run_every
         self.anchor = anchor
 
-    def remaining_estimate(self, last_run_at, now = None):
+    def remaining_estimate(self, last_run_at, now=None):
         """Returns when the periodic task should run next as a timedelta."""
-        return remaining(last_run_at, self.run_every, now = now)
+        return remaining(last_run_at, self.run_every, now=now)
 
-    def is_due(self, last_run_at, now = None):
+    def is_due(self, last_run_at, now=None):
         """Returns tuple of two items ``(is_due, next_time_to_run)``,
         where next time to run is in seconds.
 
@@ -61,7 +61,7 @@ class SchedulerEntry(object):
     '''The time and date of when this task was last run.'''
     total_run_count = None
     '''Total number of times this periodic task has been executed.'''
-    
+
     def __init__(self, name, schedule, args=(), kwargs={},
                  last_run_at = None, total_run_count=None):
         self.name = name
@@ -72,7 +72,7 @@ class SchedulerEntry(object):
     def __repr__(self):
         return self.name
     __str__ = __repr__
-    
+
     @property
     def scheduled_last_run_at(self):
         '''The scheduled last run datetime. This is different from :attr:`last_run_at` only when :attr:`anchor` is set.'''
@@ -92,15 +92,15 @@ class SchedulerEntry(object):
             return anchor
         else:
             return last_run_at
-        
+
     @property
     def run_every(self):
         return self.schedule.run_every
-    
+
     @property
     def anchor(self):
         return self.schedule.anchor
-        
+
     def next(self, now = None):
         """Returns a new instance of the same class, but with
         its date and count fields updated. Function called by :class:`Scheduler`
@@ -112,23 +112,28 @@ class SchedulerEntry(object):
 
     def is_due(self, now = None):
         return self.schedule.is_due(self.scheduled_last_run_at, now = now)
-    
+
 
 class Scheduler(object):
     """Scheduler for periodic tasks. This class is the main driver of tasks
-and task scheduling."""
-    def __init__(self, TaskFactory):
+and task scheduling.
+
+.. attribute:: task_class
+
+    The :attr:`TaskQueue.task_class` for producing new :class:`Task`.
+
+"""
+    def __init__(self, task_class):
         self._entries = self.setup_schedule()
         self.next_run = datetime.now()
-        self.TaskFactory = TaskFactory
+        self.task_class = task_class
         self.log = logging.getLogger('pulsar.tasks.scheduler')
-        
+
     @property
     def entries(self):
         return self._entries
-        
-    def queue_task(self, monitor, jobname, targs = None, tkwargs = None,
-                   **params):
+
+    def queue_task(self, monitor, jobname, targs=None, tkwargs=None, **params):
         '''Create a new :class:`Task` which may or may not queued.
 
 :parameter monitor: the :class:`pulsar.ApplicationMonitor` running the
@@ -141,7 +146,7 @@ and task scheduling."""
     in the task callable.
 :parameter params: Additional parameters to be passed to the :class:`Task`
     constructor (not its callable function).
-    
+
 :rtype: an instance of :class:`Task`'''
         task = self._make_request(jobname, targs, tkwargs, **params)
         if task.needs_queuing():
@@ -149,17 +154,14 @@ and task scheduling."""
             monitor.put(task.serialize_for_queue())
         else:
             task._queued = False
-            self.log.info('task {0} already requested. Abort request.'\
-                                  .format(task))
+            self.log.debug('Task %s already requested, abort.', task)
         return task
-    
+
     def tick(self, monitor, now=None):
         '''Run a tick, that is one iteration of the scheduler.
 Executes all due tasks calculate the time in seconds to wait before
 running a new :meth:`tick`. For testing purposes a :class:`datetime.datetime`
 value ``now`` can be passed.'''
-        # First we check for tasks which have timed out
-        self.TaskFactory.check_unready_tasks()
         remaining_times = []
         try:
             for entry in itervalues(self._entries):
@@ -168,8 +170,8 @@ value ``now`` can be passed.'''
                     self.queue_task(monitor, entry.name)
                 if next_time_to_run:
                     remaining_times.append(next_time_to_run)
-        except RuntimeError:
-            pass
+        except:
+            self.log.error('Error in task scheduler', exc_info=True)
         self.next_run = now or datetime.now()
         if remaining_times:
             self.next_run += timedelta(seconds = min(remaining_times))
@@ -184,8 +186,8 @@ value ``now`` can be passed.'''
     def setup_schedule(self):
         entries = {}
         for name, task in registry.filter_types('periodic'):
-            schedule = self.maybe_schedule(task.run_every,task.anchor)
-            entries[name] = SchedulerEntry(name,schedule)
+            schedule = self.maybe_schedule(task.run_every, task.anchor)
+            entries[name] = SchedulerEntry(name, schedule)
         return entries
 
     def job_list(self, jobnames = None):
@@ -194,74 +196,93 @@ value ``now`` can be passed.'''
             if name not in registry:
                 continue
             job = registry[name]
+            can_overlap = job.can_overlap
+            if hasattr(can_overlap, '__call__'):
+                can_overlap = 'maybe'
             d = {'doc':job.__doc__,
                  'doc_syntax':job.doc_syntax,
-                 'type':job.type}
+                 'type':job.type,
+                 'can_overlap': can_overlap}
             if name in self.entries:
                 entry = self.entries[name]
-                _,next_time_to_run = self.next_scheduled(name)
+                _,next_time_to_run = self.next_scheduled((name,))
                 run_every = 86400*job.run_every.days + job.run_every.seconds
                 d.update({'next_run':next_time_to_run,
                           'run_every':run_every,
                           'runs_count':entry.total_run_count})
             yield (name,d)
-            
-    def next_scheduled(self, jobname = None):
-        if jobname:
-            entry = self._entries.get(jobname,None)
-            if entry:
-                _, next_time_to_run = entry.is_due()
-                return (jobname,max(next_time_to_run,0))
+
+    def next_scheduled(self, jobnames=None):
+        if jobnames:
+            entries = (self._entries.get(name, None) for name in jobnames)
         else:
-            next_entry = None
-            next_time = None
-            for entry in itervalues(self._entries):
-                is_due, next_time_to_run = entry.is_due()
-                if is_due:
-                    next_time = 0
+            entries = itervalues(self._entries)
+        next_entry = None
+        next_time = None
+        for entry in entries:
+            if entry is None:
+                continue
+            is_due, next_time_to_run = entry.is_due()
+            if is_due:
+                next_time = 0
+                next_entry = entry
+                break
+            elif next_time_to_run is not None:
+                if next_time is None or next_time_to_run < next_time:
+                    next_time = next_time_to_run
                     next_entry = entry
-                    break
-                else:
-                    if next_time_to_run == None or \
-                       next_time_to_run < next_time:
-                        next_time = next_time_to_run
-                        next_entry = entry
-            if next_entry:
-                return (next_entry.name,max(next_time,0))
-            
-        return (jobname,None)
+        if next_entry:
+            return (next_entry.name, max(next_time, 0))
+        else:
+            return (jobnames, None)
+
+    def get_task(self, id, remove=False):
+        if isinstance(id, self.task_class):
+            task = id
+        else:
+            task = self.task_class.get_task(self, id)
+        if task and task.done() and remove:
+            self.delete_tasks([task.id])
+        else:
+            return task
+        
+    def get_tasks(self, **parameters):
+        return self.task_class.get_tasks(self, **parameters)
+    
+    def save_task(self, task):
+        return self.task_class.save_task(self, task)
+    
+    def delete_tasks(self, ids=None):
+        return self.task_class.delete_tasks(self, ids)
     
     ############################################################################
     ##    PRIVATE METHODS
     ############################################################################
-    
-    def _make_request(self, jobname, targs = None, tkwargs = None,
-                     expiry = None, **params):
+
+    def _make_request(self, jobname, targs=None, tkwargs=None, expiry=None,
+                      **params):
         if jobname in registry:
-            TaskFactory = self.TaskFactory
+            task_class = self.task_class
             job = registry[jobname]
             targs = targs or EMPTY_TUPLE
             tkwargs = tkwargs or EMPTY_DICT
             id = job.make_task_id(targs, tkwargs)
-            task = TaskFactory.get_task(id, remove=True)
+            task = self.get_task(id, remove=True)
             if task:
                 return task.to_queue(self)
             else:
                 if job.name in self.entries:
                     self.entries[job.name].next()
                 time_executed = datetime.now()
-                expiry = get_datetime(expiry, time_executed)
-                if not expiry and job.timeout:
-                    expiry = time_executed + job.timeout
-                task = TaskFactory(id = id, name = job.name,
-                                   time_executed = time_executed,
-                                   expiry = expiry,
-                                   args = targs,
-                                   kwargs = tkwargs,
-                                   status = PENDING,
-                                   **params)
+                if expiry is not None:
+                    expiry = get_datetime(expiry, time_executed)
+                elif job.timeout:
+                    expiry = get_datetime(job.timeout, time_executed)
+                task = task_class(id=id, name=job.name,
+                                  time_executed=time_executed, expiry=expiry,
+                                  args=targs, kwargs=tkwargs, status=PENDING,
+                                  **params)
                 task.on_created(self)
                 return task.to_queue(self)
         else:
             raise TaskNotAvailable(jobname)
-        
\ No newline at end of file
