@@ -4,32 +4,19 @@ A task scheduler application with HTTP-RPC hooks
 from datetime import datetime
 import logging
 import traceback
+from io import StringIO
 
-from pulsar.utils.py2py3 import StringIO, itervalues
-from pulsar import make_async, as_failure
+from pulsar.utils.httpurl import itervalues, iteritems
+from pulsar import maybe_async, as_failure, is_async, is_failure, send
 
+from .models import registry
 from .exceptions import *
 from .states import *
 
 
-__all__ = ['Task','TaskInMemory','TaskConsumer','nice_task_message']
+__all__ = ['Task', 'TaskInMemory', 'TaskConsumer', 'nice_task_message']
 
 
-class TaskLoggingHandler(logging.Handler):
-
-    def __init__(self, job, task):
-        self.job = job
-        self.task = task
-        super(TaskLoggingHandler,self).__init__(job.loglevel)
-        
-    def emit(self, record):
-        msg = self.format_msg(record)
-        try:
-            self.task.emit_log(record)
-        except:
-            pass
-    
-
 class TaskConsumer(object):
     '''A context manager for consuming tasks. This is used by the
 :class:`Scheduler` in a ``with`` block to correctly handle exceptions
@@ -38,94 +25,56 @@ and optional logging.
 .. attribute:: task
 
     the :class:`Task` being consumed.
-    
+
 .. attribute:: job
 
     the :class:`Job` which generated the :attr:`task`.
-    
-.. attribute:: queue
 
-    the :class:`TaskQueue` application.
-    
 .. attribute:: worker
 
     the :class:`pulsar.apps.Worker` running the process.
 '''
-    def __init__(self, task, queue, worker, job):
-        self.queue = queue
+    def __init__(self, task, worker, job):
         self.worker = worker
         self.job = job
         self.task = task
-        if job.loglevel is not None:
-            self.handler = TaskLoggingHandler(job, task)
-            formatter = job.logformatter
-            if not formatter:
-                h = logging.getLogger().handlers
-                if h:
-                    formatter = h[0].formatter
-            if formatter:
-                self.handler.setFormatter(formatter)
-        else:
-            self.handler = None
-    
-    def __enter__(self):
-        if self.handler is not None:
-            self.job.logger.addHandler(self.handler)
-        return self
-    
-    def __exit__(self, type, value, trace):
-        result = value
-        if type:
-            self.job.logger.critical(
-                        'Critical while processing task {0}'.format(self.task),
-                        exc_info = (type, value, trace))
-            result = as_failure((type, value, trace))
-        if self.handler is not None:
-            self.job.logger.removeHandler(self.handler)
-        if type:
-            return make_async(self.task.finish(self.worker,
-                                               exception = value,
-                                               result = result))\
-                    .add_callback(lambda r : result)
-        else:
-            return result
 
 
 class Task(object):
-    '''A Task interface.
+    '''Interface for tasks which are produced by :class:`Job`.
 
 .. attribute:: id
 
     :class:`Task` unique id.
-    
+
 .. attribute:: name
-    
+
     :class:`Job` name.
-    
+
 .. attribute:: status
 
     The current :ref:`status string <task-state>` of task.
-    
+
 .. attribute:: time_executed
 
     date time when the task was executed.
-    
+
 .. attribute:: time_start
 
     date-time when the task calculation has started.
-    
+
 .. attribute:: time_end
 
     date-time when the task has finished.
-    
+
 .. attribute:: expiry
 
     optional date-time indicating when the task should expire.
-    
+
 .. attribute:: timeout
-    
-    A boolean indicating whether a timeout has occurred.
-    
+
+    A datetime or ``None`` indicating whether a timeout has occurred.
+
 .. attribute:: from_task
 
     Optional :attr:`Task.id` for the :class:`Task` which queued
@@ -134,45 +83,53 @@ class Task(object):
 '''
     from_task = None
     stack_trace = None
-    
+
     @property
-    def state(self):
+    def status_code(self):
         '''Integer indicating :attr:`status` precedence.
 Lower number higher precedence.'''
-        return PRECEDENCE_MAPPING.get(self.status,UNKNOWN_STATE)
-    
-    def consumer(self, queue, worker, job):
-        '''Return the task context manager for execution.'''
-        return TaskConsumer(self, queue, worker, job)
-        
+        return PRECEDENCE_MAPPING.get(self.status, UNKNOWN_STATE)
+
     def start(self, worker):
         '''Called by the :class:`pulsar.Worker` *worker* when the task
 start its execution. If no timeout has occured the task will switch to
 a ``STARTED`` :attr:`Task.status` and invoke the :meth:`on_start`
 callback.'''
-        self.time_start = datetime.now()
-        if self.maybe_revoked():
-            self.on_timeout(worker)
-            raise TaskTimeout()
-        else:
-            self.status = STARTED
-            return self.on_start(worker)
-        
-    def finish(self, worker, exception = None, result = None):
+        job = registry[self.name]
+        try:
+            if self.maybe_revoked():
+                yield self.on_timeout(worker)
+            else:
+                self.status = STARTED
+                self.time_start = datetime.now()
+                yield self.on_start(worker)
+                consumer = TaskConsumer(self, worker, job)
+                result = maybe_async(job(consumer, *self.args, **self.kwargs))
+                if is_async(result):
+                    yield result
+                    result = maybe_async(result)
+                self.result = result
+        except Exception as e:
+            self.result = as_failure(e)
+        finally:
+            yield self.finish(worker, result=self.result)
+
+    def finish(self, worker, result):
         '''called when finishing the task.'''
         if not self.time_end:
             self.time_end = datetime.now()
-            if exception:
+            if is_failure(result):
+                result.log()
+                exception = result.trace[1]
                 self.status = getattr(exception, 'status', FAILURE)
                 self.result = str(result) if result else str(exception)
-            else:
+            # If the status is STARTED this is a succesful task
+            elif self.status == STARTED:
                 self.status = SUCCESS
                 self.result = result
-            self.on_finish(worker)
-            worker.app.broadcast('finish', self)
-        return self
-        
-    def to_queue(self, schedulter = None):
+            return self.on_finish(worker)
+
+    def to_queue(self, schedulter=None):
         '''The task has been received by the scheduler. If its status
 is PENDING swicth to RECEIVED, save the task and return it. Otherwise
 returns nothing.'''
@@ -182,104 +139,110 @@ returns nothing.'''
             self._toqueue = True
             self.on_received(schedulter)
         return self
-        
+
     def needs_queuing(self):
         '''called after calling :meth:`to_queue`, it return ``True`` if the
 task needs to be queued.'''
-        return self.__dict__.pop('_toqueue',False)
-        
+        return self.__dict__.pop('_toqueue', False)
+
     def done(self):
         '''Return ``True`` if the task has its staus in READY_STATES'''
         return self.status in READY_STATES
-    
-    def maybe_revoked(self):
-        '''Try to revoke a task. If succesful return the expiry.
 
-:rtype: ``None`` or a DateTime instance'''
-        if self.state > PRECEDENCE_MAPPING[STARTED] and self.expiry:
-            tm = datetime.now()
-            if tm > self.expiry:
-                self.status = REVOKED
-                self.timeout = tm
-                return tm
-        
+    def maybe_revoked(self):
+        '''Try to revoke a task. It returns the :attr:`timeout` which is
+different from ``None`` only when the :class:`Task` has been revoked.
+
+:rtype: ``None`` or a DateTime.'''
+        if not self.timeout:
+            if self.status_code > PRECEDENCE_MAPPING[STARTED] and self.expiry:
+                tm = datetime.now()
+                if tm > self.expiry:
+                    self.status = REVOKED
+                    self.timeout = tm
+        return self.timeout
+
     def execute2start(self):
         if self.time_start:
             return self.time_start - self.time_executed
-        
+
     def execute2end(self):
         if self.time_end:
             return self.time_end - self.time_executed
-        
+
     def duration(self):
-        if self.time_end:
-            return self.time_end - self.time_start  
+        '''The :class:`Task` duration. Only available if the task status is in
+:attr:`FULL_RUN_STATES`.'''
+        if self.time_end and self.time_start:
+            return self.time_end - self.time_start
 
     def tojson(self):
         '''Convert the task instance into a JSON-serializable dictionary.'''
         return self.__dict__.copy()
-    
-    def on_same_id(self):
-        self.delete()
-    
+
     def serialize_for_queue(self):
         return self
-    
+
     def ack(self):
         return self
-    
+
     ############################################################################
     ##    FACTORY METHODS
-    
+    ############################################################################
     @classmethod
-    def from_queue(cls, task):
-        return task
-    
-    @classmethod
-    def get_task(cls, id, remove = False):
+    def get_task(cls, scheduler, id):
         '''Given a task *id* it retrieves a task instance or ``None`` if
 not available.'''
         raise NotImplementedError()
     
     @classmethod
-    def check_unready_tasks(cls):
-        '''Check for expiries in all tasks in an un-ready state.'''
+    def get_tasks(cls, scheduler, **filters):
+        '''Given *filters* it retrieves task instances which satisfy the
+filter criteria.'''
         raise NotImplementedError()
     
+    @classmethod
+    def save_task(cls, scheduler, task):
+        raise NotImplementedError()
+        
+    @classmethod
+    def delete_tasks(cls, scheduler, task):
+        raise NotImplementedError()
+
     ############################################################################
     # CALLBACKS
     ############################################################################
-    
-    def on_created(self, scheduler = None):
+
+    def on_created(self, scheduler=None):
         '''A :ref:`task callback <tasks-callbacks>` when the task has
 has been created.
 
 :parameter scheduler: the scheduler which created the task.
 '''
         pass
-    
-    def on_received(self, scheduler = None):
+
+    def on_received(self, scheduler=None):
         '''A :ref:`task callback <tasks-callbacks>` when the task has
 has been received by the scheduler.'''
         pass
-        
-    def on_start(self, worker = None):
+
+    def on_start(self, worker=None):
         '''A :ref:`task callback <tasks-callbacks>` when the task starts
 its execution'''
         pass
-    
+
     def on_timeout(self, worker=None):
         '''A :ref:`task callback <tasks-callbacks>` when the task is expired'''
         pass
 
-    def on_finish(self, worker = None):
+    def on_finish(self, worker=None):
         '''A :ref:`task callback <tasks-callbacks>` when the task finish
 its execution'''
         pass
 
     def close(self):
         pass
-    
+
     def emit_log(self, record):
         '''Implement the task logging emit method. By default it does nothing.
 It can be reimplemented to do something with the log record.'''
@@ -287,17 +250,16 @@ It can be reimplemented to do something with the log record.'''
 
 
 class TaskInMemory(Task):
-    '''An in memory implementation of a Task'''
-    _TASKS = {}
+    '''An in memory implementation of a Task.'''
     time_start = None
     time_end = None
     stack_trace = None
     result = None
     timeout = False
-    
-    def __init__(self, id = None, name = None, time_executed = None,
-                 expiry = None, args = None, kwargs = None, ack = None,
-                 status = None, from_task = None, **params):
+
+    def __init__(self, id=None, name=None, time_executed=None,
+                 expiry=None, args=None, kwargs=None, ack=None,
+                 status=None, from_task=None, **params):
         self.id = id
         self.name = name
         self.time_executed = time_executed
@@ -307,62 +269,82 @@ class TaskInMemory(Task):
         self.status = status
         self.from_task = from_task
         self.params = params
-        
-    def __str__(self):
-        return '{0}({1})'.format(self.name,self.id)
 
-    def on_received(self,worker=None):
+    def __repr__(self):
+        return '%s(%s)' % (self.name,self.id)
+    __str__ = __repr__
+
+    def on_received(self, scheduler=None):
         # Called by the scheduler
-        self.__class__.save_task(self)
-    
+        self.save_task(scheduler, self)
+
     def on_start(self, worker=None):
         if worker:
-            return worker.monitor.send(worker,'save_task',self)
-        
+            return send(worker.monitor, 'save_task', self)
+
     def on_timeout(self, worker=None):
         if worker:
-            return worker.monitor.send(worker,'save_task',self)
-                    
-    def on_finish(self,worker=None):
+            return send(worker.monitor, 'save_task', self)
+
+    def on_finish(self, worker=None):
         if worker:
-            return worker.monitor.send(worker, 'save_task', self)
-        
-    def delete(self):
-        self._TASKS.pop(self.id,None)
-        
+            return send(worker.monitor, 'save_task', self)
+
     @classmethod
-    def save_task(cls, task):
-        if task.id in cls._TASKS:
-            t = cls._TASKS[task.id]
-            if t.state < task.state:
+    def task_container(cls, scheduler):
+        if not hasattr(scheduler, '_TASKS'):
+            scheduler._TASKS = {}
+        return scheduler._TASKS
+    
+    @classmethod
+    def save_task(cls, scheduler, task):
+        TASKS = cls.task_container(scheduler)
+        if task.id in TASKS:
+            t = TASKS[task.id]
+            if t.status_code < task.status_code:
                 # we don't save here. Could by concurrency lags
                 return
-        cls._TASKS[task.id] = task
-    
-    @classmethod
-    def get_task(cls, id, remove=False):
-        task = cls._TASKS.get(id, None)
-        if remove and task:
-            if task.done():
-                cls._TASKS.pop(id)
-        return task
-    
-    @classmethod
-    def check_unready_tasks(cls):
-        tasks = cls._TASKS
-        for task in list(itervalues(tasks)):
-            task.maybe_revoked()
-            if task.done():
-                tasks.pop(task.id,None)
-            
+        TASKS[task.id] = task
 
-def nice_task_message(req, smart_time = None):
+    @classmethod
+    def delete_tasks(cls, scheduler, ids=None):
+        TASKS = cls.task_container(scheduler)
+        if ids:
+            for id in ids:
+                TASKS.pop(id, None)
+        else:
+            TASKS.clear()
+        
+    @classmethod
+    def get_task(cls, scheduler, id):
+        return cls.task_container(scheduler).get(id)
+    
+    @classmethod
+    def get_tasks(cls, scheduler, **filters):
+        TASKS = cls.task_container(scheduler)
+        tasks = []
+        if filters:
+            fs = []
+            for name, value in iteritems(filters):
+                if not isinstance(value, (list, tuple)):
+                    value = (value,)
+                fs.append((name, value))
+            for t in itervalues(TASKS):
+                for name, values in fs:
+                    value = getattr(t, name, None)
+                    if value in values:
+                        tasks.append(t)
+        return tasks
+
+
+def format_time(dt):
+    return dt.isoformat() if dt else '?'
+
+def nice_task_message(req, smart_time=None):
     smart_time = smart_time or format_time
     status = req['status'].lower()
-    user = req.get('user',None)
-    ti = req.get('time_start',req.get('time_executed',None))
-    name = '{0} ({1}) '.format(req['name'],req['id'][:8])
-    msg = '{0} {1} at {2}'.format(name,status,smart_time(ti))
-    if user:
-        msg = '{0} by {1}'.format(msg,user)
-    return msg
\ No newline at end of file
+    user = req.get('user')
+    ti = req.get('time_start', req.get('time_executed'))
+    name = '%s (%s) ' % (req['name'], req['id'][:8])
+    msg = '%s %s at %s' % (name, status, smart_time(ti))
+    return '%s by %s' % (msg, user) if user else msg
