@@ -5,11 +5,15 @@ import logging
 from inspect import istraceback
 from copy import deepcopy
 
+from pulsar.utils.structures import AttributeDictionary
 
-__all__ = ['TestObject',
+from .utils import TestFunction
+
+__all__ = ['Plugin',
            'TestStream',
            'TestRunner',
-           'TestResult']
+           'TestResult',
+           'Plugin']
 
 
 LOGGER = logging.getLogger('pulsar.apps.test')
@@ -17,12 +21,12 @@ STDOUT_LINE = '\nStdout:\n%s'
 STDERR_LINE = '\nStderr:\n%s'
 
 
-class TestObject(object):
+class Plugin(object):
     '''Interface for all classes which are part of of the :class:`TestRunner`,
 including :class:`TestRunner` itself, :class:`TestResult`
 and :class:`Plugin`.'''
     descriptions = None
-    
+
     def configure(self, cfg):
         '''Configure the *instance*. This method is called once just after
 construction of a :class:`TestRunner` and can be used to configure the
@@ -34,50 +38,63 @@ it will stop the configuration of all subsequent plugins and quit the test.
 '''
         pass
     
+    @property
+    def name(self):
+        return self.__class__.__name__.lower()
+
     def loadTestsFromTestCase(self, cls):
         pass
-    
+
     def on_start(self):
         '''Called once by :class:`TestSuite` just before it starts running tests.'''
         pass
-    
+
     def on_end(self):
         '''Called once by :class:`TestSuite` just before it stops.'''
         pass
-    
+
     def startTest(self, test):
+        '''Called just before a *test* is executed. This is run
+just before _pre_setup function.'''
+        pass
+
+    def stopTest(self, test):
+        '''Called just after a *test* has finished. This is run just after
+the _post_teardown function.'''
         pass
     
-    def getTest(self, test):
-        '''Given a test instance return a, possibly, modified test
-instance. This function can be used by plugins to modify the behaviour of test
-cases. By default it returns *test*.'''
+    def before_test_function_run(self, test, local):
+        '''This function can be used by plugins to manipulate the *test*
+behaviour in the process domain where the test run.'''
         return test
     
-    def stopTest(self, test):
+    def after_test_function_run(self, test, local, result):
+        '''Given a test-function instance return a, possibly, modified test
+instance. This function can be used by plugins to modify the behaviour of test
+cases. By default it returns *test*.'''
         pass
-    
+
     def addSuccess(self, test):
         pass
-    
+
     def addFailure(self, test, err):
         pass
-    
+
     def addError(self, test, err):
         pass
-    
+
     def addSkip(self, test, reason):
         pass
-    
+
     def printErrors(self):
         pass
-    
+
     def printSummary(self, timeTaken):
         pass
-    
+
     def import_module(self, mod, parent = None):
         return mod
-    
+
     def getDescription(self, test):
         doc_first_line = test.shortDescription()
         teststr = '{0}.{1}'.format(test.tag,test)
@@ -85,57 +102,61 @@ cases. By default it returns *test*.'''
             return '\n'.join((teststr, doc_first_line))
         else:
             return teststr
-    
-    
-class TestResultProxy(TestObject):
+
+
+class TestResultProxy(Plugin):
     result = None
     stream = None
-    
+
     @property
     def count(self):
         return self.result.count if self.result else 0
-    
+
     @property
     def testsRun(self):
         return self.result.testsRun if self.result else 0
-    
+
 
 class TestStream(TestResultProxy):
+    '''handle the writing of test results'''
     separator1 = '=' * 70
     separator2 = '-' * 70
-    
-    def __init__(self, stream, result, descriptions = True):
+
+    def __init__(self, stream, result, descriptions=True):
         self._handlers = {}
         self.stream = stream
         self.result = result
         self.descriptions = descriptions
         self.showAll = False
         self.dots = True
-        
+
     def configure(self, cfg):
         verbosity = cfg.verbosity
         self.showAll = verbosity > 1
         self.dots = verbosity == 1
-        
+
     def handler(self, name):
         return self._handlers.get(name,self.stream)
-    
+
     def startTest(self, test):
-        pass
-    
-    def head(self, test, v):
         if self.showAll:
-            v = self.getDescription(test) + ' ... ' + v + '\n'
+            v = self.getDescription(test) + ' ... '
             self.stream.write(v)
             self.stream.flush()
-            
+
+    def head(self, test, v):
+        #v = self.getDescription(test) + ' ... ' + v + '\n'
+        v += '\n'
+        self.stream.write(v)
+        self.stream.flush()
+
     def addSuccess(self, test):
         if self.showAll:
             self.head(test,'ok')
         elif self.dots:
             self.stream.write('.')
             self.stream.flush()
-            
+
     def addError(self, test, err):
         if self.showAll:
             self.head(test,"ERROR")
@@ -221,14 +242,14 @@ class TestStream(TestResultProxy):
             stream.writeln(" (%s)" % (", ".join(infos),))
         else:
             stream.write("\n")
-            
+
         return True
 
 
-class TestResult(TestObject):
-    
-    def __init__(self, descriptions = True):
-        self.descriptions = descriptions 
+class TestResult(Plugin):
+
+    def __init__(self, descriptions=True):
+        self.descriptions = descriptions
         self.testsRun = 0
         self._count = 0
         self.failures = []
@@ -236,40 +257,40 @@ class TestResult(TestObject):
         self.skipped = []
         self.expectedFailures = []
         self.unexpectedSuccesses = []
-    
+
     @property
     def count(self):
         return self._count
-    
+
     def startTest(self, test):
         self.testsRun += 1
-    
+
     def addError(self, test, err):
         """Called when an error has occurred. 'err' is a tuple of values as
         returned by sys.exc_info().
         """
         self._add_error(test, err, self.errors)
-        
+
     def addFailure(self, test, err):
         self._add_error(test, err, self.failures)
-        
+
     def addSkip(self, test, reason):
         """Called when a test is skipped."""
         self.skipped.append((self.getDescription(test), reason))
-        
+
     def addExpectedFailure(self, test, err):
         """Called when an expected failure/error occured."""
         self._add_error(test, err, self.expectedFailures)
-        
+
     def addUnexpectedSuccess(self, test):
         """Called when a test was expected to fail, but succeed."""
         self.unexpectedSuccesses.append(self.getDescription(test))
-    
+
     def _add_error(self, test, err, container):
         err = self._exc_info_to_string(err, test)
         test = self.getDescription(test)
         container.append((test, err))
-                
+
     def add(self, result):
         self._count += 1
         self.testsRun += result.testsRun
@@ -278,11 +299,11 @@ class TestResult(TestObject):
         self.skipped.extend(result.skipped)
         self.expectedFailures.extend(result.expectedFailures)
         self.unexpectedSuccesses.extend(result.unexpectedSuccesses)
-        
+
     def wasSuccessful(self):
         "Tells whether or not this result was a success"
         return len(self.failures) == len(self.errors) == 0
-        
+
     def _exc_info_to_string(self, err, test):
         """Converts a sys.exc_info()-style tuple of values into a string."""
         exctype, value, tb = err
@@ -290,7 +311,7 @@ class TestResult(TestObject):
         if istraceback(tb):
             while tb and self._is_relevant_tb_level(tb):
                 tb = tb.tb_next
-    
+
             if exctype is test.failureException:
                 # Skip assert*() traceback levels
                 length = self._count_relevant_tb_levels(tb)
@@ -298,7 +319,13 @@ class TestResult(TestObject):
             else:
                 msgLines = traceback.format_exception(exctype, value, tb)
         else:
-            msgLines = tb or []
+            msgLines = tb
+
+        if not msgLines:
+            if value:
+                msgLines = [repr(value)]
+            else:
+                msgLines = ()
 
         return ''.join(msgLines)
 
@@ -316,12 +343,12 @@ class TestResult(TestObject):
     
 class TestRunner(TestResultProxy):
     '''An asynchronous test runner'''
-    def __init__(self, plugins, stream, writercls = None, descriptions=True):
+    def __init__(self, plugins, stream, writercls=None, descriptions=True):
         self.descriptions = descriptions
         self.plugins = []
         writercls = writercls or TestStream
-        result = TestResult(descriptions = self.descriptions)
-        stream = writercls(stream, result, descriptions = self.descriptions)
+        result = TestResult(descriptions=self.descriptions)
+        stream = writercls(stream, result, descriptions=self.descriptions)
         for p in plugins:
             p = deepcopy(p)
             p.descriptions = self.descriptions
@@ -333,28 +360,29 @@ class TestRunner(TestResultProxy):
         self.stream = stream
         self.result = result
         self.loader = unittest.TestLoader()
-        
+
     def configure(self, cfg):
         self.cfg = cfg
         for p in self.plugins:
             c = p.configure(cfg)
             if c:
-                return c 
-    
+                return c
+
     def loadTestsFromTestCase(self, cls):
+        '''Load all *test* functions for the test class *cls*.'''
         all_tests = self.loader.loadTestsFromTestCase(cls)
         for p in self.plugins:
             c = p.loadTestsFromTestCase(cls)
             if c is not None:
                 return c
         return all_tests
-        
+
     def on_start(self):
         '''Called just before the test suite starts running tests.'''
         for p in self.plugins:
             if p.on_start():
                 break
-            
+
     def on_end(self):
         '''Called just before the test suite starts running tests.'''
         for p in self.plugins:
@@ -363,62 +391,87 @@ class TestRunner(TestResultProxy):
                     break
             except:
                 LOGGER.critical('Unhandled exception while calling method'\
-                                ' "on_end" of plugin {0}'.format(p),
-                                exc_info=True)
-        
+                                ' "on_end" of plugin %s', p, exc_info=True)
+
     def add(self, result):
         self.result.add(result)
-    
-    def import_module(self, mod, parent = None):
+
+    def import_module(self, mod, parent=None):
         for p in self.plugins:
-            mod = p.import_module(mod,parent)
+            mod = p.import_module(mod, parent)
             if not mod:
                 return
         return mod
-    
+
     def startTest(self, test):
         '''Called before test has started.'''
         for p in self.plugins:
             if p.startTest(test):
                 break
-            
-    def getTest(self, test):
+
+    def before_test_function_run(self, test):
+        '''Called before the test run, in the test process domain.'''
+        test.plugins = plugins = {}
         for p in self.plugins:
-            test = p.getTest(test) or test
+            local = AttributeDictionary()
+            plugins[p.name] = local
+            test = p.before_test_function_run(test, local) or test
         return test
-            
+    
+    def after_test_function_run(self, test, result):
+        '''Called before the test starts.'''
+        for p in self.plugins:
+            local = test.plugins.get(p.name)
+            if local is not None:
+                p.after_test_function_run(test, local, result)
+        return result
+
     def stopTest(self, test):
         '''Called after test has finished.'''
         for p in self.plugins:
             if p.stopTest(test):
                 break
-            
+
     def addSuccess(self, test):
         for p in self.plugins:
             if p.addSuccess(test):
                 break
-            
+
     def addFailure(self, test, err):
         for p in self.plugins:
             if p.addFailure(test, err):
                 break
-            
+
     def addError(self, test, err):
         for p in self.plugins:
             if p.addError(test, err):
                 break
-            
+
     def addSkip(self, test, reason):
         for p in self.plugins:
             if p.addSkip(test, reason):
                 break
-    
+
     def printErrors(self):
         for p in self.plugins:
             if p.printErrors():
                 break
-    
+
     def printSummary(self, timeTaken):
         for p in self.plugins:
             if p.printSummary(timeTaken):
-                break
\ No newline at end of file
+                break
+            
+    def run_test_function(self, test, func):
+        '''Run function *func* which belong to *test*.
+    
+:parameter test: test instance or class
+:parameter func: test function belonging to *test*
+:return: an asynchronous result
+'''
+        if func is None:
+            return func
+        tfunc = getattr(func, 'testfunction', None)
+        if tfunc is None:
+            tfunc = TestFunction(func.__name__)
+        return tfunc(test)
\ No newline at end of file
