@@ -33,8 +33,7 @@ def import_modules(modules):
             except ImportError as e:
                 pass
             
-
-def module_attribute(dotpath, default = None, safe = False):
+def module_attribute(dotpath, default=None, safe=False):
     '''
     Load an attribute from a module.
     If the module or the attriubute is not available,
@@ -53,3 +52,29 @@ def module_attribute(dotpath, default = None, safe = False):
         if not safe:
             raise ImportError()
         return default
+
+def py_file(name):
+    if name.endswith('.py'):
+        return name[:-3]
+    elif name.endswith('.pyc'):
+        return name[:-4]
+    else:
+        return name
+    
+def import_system_file(mod, add_to_path=True):
+    if os.path.isfile(mod):
+        # it is a file in the system path
+        dir, name = os.path.split(mod)
+        names = [py_file(name)]
+        while dir and not dir in sys.path:
+            dir, name = os.path.split(dir)
+            names.insert(0, name)
+        # the file was not in the system path
+        if not dir and add_to_path:
+            dir, name = os.path.split(mod)
+            if dir:
+                sys.path.append(dir)
+            mod_name = py_file(name)
+        else:
+            mod_name = '.'.join(names)
+        return import_module(mod_name)
